package
{	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.ui.Mouse;
	
	import mx.utils.StringUtil;
	
	import org.flixel.*;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;
	import org.flixel.FlxU;
	
	public class GameState extends FlxState
	{
		[Embed(source = "data/bg.png")] public static var ImgBackDrop:Class;
		[Embed(source = "data/player.png")] public static var ImgPlayer:Class;
		[Embed(source = "data/ghost.png")] public static var ImgGhost:Class;
		[Embed(source = "data/camera.png")] public static var ImgCamera:Class;
		[Embed(source = "data/map.png")] public static var ImgMap:Class;
		[Embed(source = "data/target.png")] public static var ImgTarget:Class;
		[Embed(source = "data/animal.png")] public static var ImgAntelope:Class;
		[Embed(source = "data/deadanimal.png")] public static var ImgRoadkill:Class;
		[Embed(source = "data/tree.png")] public static var ImgTree:Class;
		
		[Embed(source = "data/music.mp3")] public static var SndMusic:Class;
		[Embed(source = "data/pushblock.mp3")] public static var SndBlock:Class;
		[Embed(source = "data/camera.mp3")] public static var SndCamera:Class;
		
		[Embed(source = "data/level1.txt", mimeType="application/octet-stream")] public var Level0:Class;
		[Embed(source = "data/level2.txt", mimeType="application/octet-stream")] public var Level1:Class;
		[Embed(source = "data/level3.txt", mimeType="application/octet-stream")] public var Level2:Class;
		[Embed(source = "data/level4.txt", mimeType="application/octet-stream")] public var Level3:Class;
		[Embed(source = "data/level5.txt", mimeType="application/octet-stream")] public var Level4:Class;
		[Embed(source = "data/level6.txt", mimeType="application/octet-stream")] public var Level5:Class;

		public var gamemanager:GameManager;
		public var player:Player;	
		public var enemies:FlxGroup;
		public var coins:FlxGroup;
		public var tilemap:FlxTilemap;
		public var gridsprites:Vector.<GridSprite>;
		public var camerasprites:Vector.<GridSprite>;
		public var cratesprites:Vector.<GridSprite>;
		public var exitsprites:Vector.<GridSprite>;
		public var gridsprite_bg_group:FlxGroup;
		public var gridspritegroup:FlxGroup;
		
		public var cameras:Vector.<FlxCamera>;		
		
		public function GetSpritesAt(px:int,py:int):Vector.<GridSprite>
		{
			var result:Vector.<GridSprite> = new Vector.<GridSprite>();
			for(var i:int=0;i<gridsprites.length;i++)
			{
				var g:GridSprite = gridsprites[i];
				if (g.posx==px&&g.posy==py)
					result.push(g);
			}
			return result;
		}
		
		//ready for movement
		public function Ready():Boolean
		{
			for(var i:int=0;i<gridsprites.length;i++)
			{
				if (gridsprites[i].ready==false)
					return false;
			}
			return true;
		}
		
		public function IsWallAt(px:int,py:int):Boolean
		{
			return gamemanager.map[py][px]=="#";
		}
		
		public function UpdateGameManager():void
		{
			//step one, remove all sprite/&c. references from game manager
			for (var i:int=0;i<gamemanager.map.length;i++)
			{
				for (var j:int=0;j<gamemanager.map[i].length;j++)
				{
					if (gamemanager.map[i][j]!="#" && gamemanager.map[i][j]!="X")
					{
						gamemanager.map[i][j]=" ";
					}					
				}
			}
			
			for (i=0;i<gridsprites.length;i++)
			{
				var gs:GridSprite=gridsprites[i];
				gamemanager.PrintSprite(gs);
			}
			
			trace("REALITY");
			trace(gamemanager.Serialize());
		}
		
		public function printSolution():void
		{						
			UpdateGameManager();
						
			var gm:GameManager = new GameManager(true);
			
			var ghostcount:int=camerasprites.length;//FlxG.level+1;
			var across:int = int(Math.sqrt(ghostcount));
			if (across==0)
				across=1;
			var down:int = ghostcount/across;
			if (down==0)
				down=1;
			ghostcount=across*down;
			
			var camwidth:int=FlxG.width/across;
			camwidth-=camwidth%8;
			camwidth/=8;
			var camheight:int=FlxG.height/down;
			camheight-=camheight%8;
			camheight/=8;
			//var offsetx:Boolean = (camwidth/8) %2 == 0;
			//var offsety:Boolean = (camheight/8) %2 == 0;
			var camoffsetx:int=camwidth/2;
			var camoffsety:int=camheight/2;
			for (var i:int=0;i<across;i++)
			{
				for (var j:int=0;j<down;j++)
				{
					//position of camera on-screen
					var camx:int=camwidth*i;
					var camy:int=camheight*j;
					
					//position of camera in world
					var camsourcex:int=camerasprites[i*down+j].posx-camoffsetx;
					var camsourcey:int=camerasprites[i*down+j].posy-camoffsety;
					
					//copy blocks from global to gm
					for (var tx:int=0;tx<camwidth;tx++)
					{
						for (var ty:int=0;ty<camheight;ty++)
						{
							/*
							block position tx,ty corresponds to world position
							world position: camsourcex+tx, camsourcey+ty
							screen position: tx+camx,ty+camy
							//screen coords in range, world coords might be outside
							*/
							var worldposx:int=camsourcex+tx;
							var worldposy:int=camsourcey+ty;
							if (worldposx<0 || worldposx>=gamemanager.map[0].length)
								continue;
							if (worldposy<0 || worldposy>=gamemanager.map.length)
								continue;
							
							var screenposx:int=camx+tx;
							var screenposy:int=camy+ty;
							gm.map[screenposy][screenposx]=gamemanager.map[worldposy][worldposx];
						}
					}
				}
			}
			
			trace("VISIBLE");
			trace(gm.Serialize());
		}
		
		override public function create():void
		{		
			Player.levelending=false;
			super.create();
			gamemanager= new GameManager();
			//var bd:FlxSprite = new FlxSprite(0,0,ImgBackDrop);
			//add(bd);
			FlxG.bgColor=0xff000000;
			gridsprites = new Vector.<GridSprite>();
			cratesprites = new Vector.<GridSprite>();
			exitsprites = new Vector.<GridSprite>();
			camerasprites = new Vector.<GridSprite>();
			var mapstring:String = new this["Level"+FlxG.level.toString()]();
			var tilemapstring:String = "";
			var rows : Array = mapstring.split("\n");
			
			for (i=0;i<rows.length;i++)
			{
				rows[i]="#######"+mx.utils.StringUtil.trim(rows[i])+"#######";
			}
			
			var blankrow:String="";
			var r1:String = rows[0];
			for(i=0;i<r1.length+7*2;i++)
			{
				blankrow+="#";
			}
			
			var newrows:Array = [];
			for (i=0;i<7;i++)
			{
				newrows.push(blankrow);
			}
			for(i=0;i<rows.length;i++)
			{
				newrows.push(rows[i]);
			}
			for (i=0;i<7;i++)
			{
				newrows.push(blankrow);
			}
			rows=newrows;
			
			
			var tilemapGroup:FlxGroup =new FlxGroup();
			add(tilemapGroup);
			gridsprite_bg_group = new FlxGroup();
			add(gridsprite_bg_group);
			gridspritegroup = new FlxGroup();
			add(gridspritegroup);
			
			for (var i:int=0;i<rows.length;i++)
			{
				if (i>0)					
					tilemapstring+="\n";
				var chars:String = rows[i];//mx.utils.StringUtil.trim(rows[i]);
				var row:Vector.<String> = new Vector.<String>();
				
				var leftboundary:int = chars.indexOf("#",0);
				var rightboundary:int = chars.length-1 - chars.split("").reverse().join("").indexOf("#",0);
				
				for (var j:int=0;j<chars.length;j++)
				{
					if (j>0)
						tilemapstring+=",";
					
					
					var interior:Boolean=j>=leftboundary && j<=rightboundary;
					
					var c:String=chars.charAt(j);
					
					if (interior==false && c==" ")
					{
						c="X";
					}
					
					row.push(c);

					switch(c)
					{
						case "#":
							tilemapstring+="2";
							break;
						case "X":
							tilemapstring+="3";
							break;
						default:
							tilemapstring+="1";
					}
					
					
					switch(c)
					{
						case "a":
						{				
							var cam:Camera = new Camera(j,i,8,GameState.ImgCamera,this);
							camerasprites.push(cam);
							gridsprites.push(cam);
							gridspritegroup.add(cam);
							break;
						}
						case "P":
						{						
							player = new Player(j,i,8,GameState.ImgPlayer,this);
							gridsprites.push(player);
							gridspritegroup.add(player);
							break;
						}
						case "B":
						{						
							player = new Player(j,i,8,GameState.ImgPlayer,this);
							gridsprites.push(player);
							gridspritegroup.add(player);
							t = new Target(j,i,8,GameState.ImgTarget,this);
							exitsprites.push(t);
							gridsprites.push(t);
							gridsprite_bg_group.add(t);
							break;
						}	
						case "$":
						{						
							var g:Block = new Block(j,i,8,GameState.ImgPlayer,this);
							cratesprites.push(g);
							gridsprites.push(g);
							gridspritegroup.add(g);
							break;
						}	
						case "@":
						{						
							cam = new Camera(j,i,8,GameState.ImgCamera,this);
							camerasprites.push(cam);
							gridsprites.push(cam);
							gridspritegroup.add(cam);
							t = new Target(j,i,8,GameState.ImgTarget,this);
							exitsprites.push(t);
							gridsprites.push(t);
							gridsprite_bg_group.add(t);
							break;
						}		
						case ".":
						{						
							var t:Target = new Target(j,i,8,GameState.ImgTarget,this);
							exitsprites.push(t);
							gridsprites.push(t);
							gridsprite_bg_group.add(t);
							break;
						}						
					}
				}
				gamemanager.map.push(row);
			}
			
			for (i=0;i<gridsprites.length;i++)
			{
				gridsprites[i].tweenComplete();
			}
			//FlxG.camera.visible=false;
			var maincameradistraction:FlxObject = new FlxObject(-1000,-100,10,10);
			FlxG.camera.follow(maincameradistraction);
			FlxG.camera.bgColor=0xff000000;
			//FlxG.camera.width=200;
			//FlxG.camera.height=200;
			enemies = new FlxGroup();
			add(enemies);
			
			coins = new FlxGroup();
			add(coins);			
			
			trace("TMS\n"+tilemapstring);
			tilemap = new FlxTilemap();
			tilemap.loadMap(tilemapstring,ImgMap,8,8);
			tilemap.collideIndex=2;
			tilemapGroup.add(tilemap);
			//tilemap.visible=false;
			
			cameras = new Vector.<FlxCamera>();
			
			//FlxG.removeAllCameras();
			
			var ghostcount:int=camerasprites.length;//FlxG.level+1;
			var across:int = int(Math.sqrt(ghostcount));
			if (across==0)
				across=1;
			var down:int = ghostcount/across;
			if (down==0)
				down=1;
			ghostcount=across*down;
			
			var camwidth:int=FlxG.width/across;
			camwidth-=camwidth%8;
			var camheight:int=FlxG.height/down;
			camheight-=camheight%8;
			var offsetx:Boolean = (camwidth/8) %2 == 0;
			var offsety:Boolean = (camheight/8) %2 == 0;
			for (i=0;i<across;i++)
			{
				for (j=0;j<down;j++)
				{
					var ghost:GridSprite = camerasprites[i*down+j];
									
					var camera:FlxCamera = new FlxCamera(
							8*i*camwidth,
							8*j*camheight,
							camwidth,
							camheight,
							8);
					camera.bgColor=0xff444444;
					//camera.bounds = new FlxRect(0,0,tilemap.width,tilemap.height);
					//camera.offset = new FlxPoint(offsetx?-4:0,offsety?-4:0);
					//camera.follow(ghost);
					camera.target=ghost;
					cameras.push(camera);
					//camera.antialiasing
					//camera.visible=false;
					FlxG.addCamera(camera);
				}
			}
			
		}		
		
		override public function update():void
		{			
			for (i=0;i<cameras.length;i++)
			{
//				cameras[i].shake(0.005,100000000);
			}
			
			var checked:Boolean=true;
			if (Ready())
			{
				for (i=0;i<gridsprites.length;i++)
				{
					if (gridsprites[i].checked==false)
					{
						checked=false;
						break;
					}
				}
				if (checked==false)
				{				
					this.printSolution();
					for (i=0;i<gridsprites.length;i++)
					{
						gridsprites[i].checked=true;
					}
				}
			}
				
			for (i=0;i<cameras.length;i++)
			{
				//cameras[i].bgColor=0xffffffff;
				//cameras[i].setScale(Math.random()+0.5,Math.random()+0.5);
				//cameras[i].angle=Math.random()*360;
			}
			
			if (FlxG.keys.justPressed("C"))
			{
				FlxG.camera.visible=cameras[0].visible;
				for (var i:int=0;i<cameras.length;i++)
				{
					cameras[i].visible=!cameras[i].visible;
				}
			}
			super.update();
		}					
		
	}
}