package
{
	import com.greensock.TweenLite;
	
	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	
	public class GridSprite extends FlxSprite
	{
		//whether moving or at rest
		public var ready:Boolean=true;
		public var checked:Boolean=false;
		
		private var _tilesize:int=0;
		public var posx:int=0;
		public var posy:int=0;
		
		private var _targetx:int=0;
		private var _targety:int=0;
		
		private const _movetime:Number=0.2;
		protected var _parent:GameState;
		
		public function GridSprite(tilePosX:int,tilePosY:int,tilesize:int,Graphic:Class,parent:GameState)
		{
			posx=tilePosX;
			posy=tilePosY;		
			_tilesize=tilesize;
			_parent=parent;
			super(0, 0, Graphic);
			refreshposition();	
		}
		
		public function CanPush(dir:uint):Boolean
		{
			var tx:int=posx;
			var ty:int=posy;
			switch(dir)
			{
				case FlxObject.UP:
					ty--;
					break;
				case FlxObject.DOWN:
					ty++;
					break;
				case FlxObject.LEFT:
					tx--;
					break;
				case FlxObject.RIGHT:
					tx++;
					break;
			}
			var s:Vector.<GridSprite> = _parent.GetSpritesAt(tx,ty);
			if (s.length>0)
			{
				var success:Boolean=true;
				for (var i:int=0;i<s.length;i++)
				{
					success = success && s[i].CanPush(dir);
				}
				return success;
			}
			else
			{
				return _parent.IsWallAt(tx,ty)==false;
			}
		}
		
		private function refreshposition():void
		{
			TweenLite.killTweensOf(this);
			ready=true;
			x=posx*_tilesize;
			y=posy*_tilesize;				
		}
		
		//assumes can already push
		public function Push(dir:uint):void
		{
			var pushed:Boolean=false;
				var tx:int=posx;
				var ty:int=posy;
				switch(dir)
				{
					case FlxObject.UP:
						ty--;
						break;
					case FlxObject.DOWN:
						ty++;
						break;
					case FlxObject.LEFT:
						tx--;
						break;
					case FlxObject.RIGHT:
						tx++;
						break;
				}
				var s:Vector.<GridSprite> = _parent.GetSpritesAt(tx,ty);
				if (s.length>0)
				{
					pushed=true;
					var success:Boolean=true;
					for (var i:int=0;i<s.length;i++)
					{
						s[i].Push(dir);
					}
				}
			
			move(tx,ty,posx,posy);	
		}
		
		//passing -1 as fromx/y results in immediate snapping;
		public function move(tox:int,toy:int,fromx:int=-1,fromy:int=-1):void
		{
			if (fromx==-1)
			{
				posx=tox;
				posy=toy;
				refreshposition();
			}			
			else
			{
				posx=fromx;
				posy=fromy;
				_targetx=tox;
				_targety=toy;
				refreshposition();
				posx=tox;
				posy=toy;
				ready=false;
				TweenLite.to(this,_movetime,{x:_targetx*_tilesize,y:_targety*_tilesize,onComplete:tweenComplete});
			}
		}
				
		public function tweenComplete():void
		{
			refreshposition();
			ready=true;
			checked=false;
		}
			
		override public function update():void
		{
			
			super.update();
		}
		
	}
	
}